==> matrix_3.2.tgz: matrix/matrix_saver.c <==
/*-
 * Copyright (c) 1999 Andrzej Bialecki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id$
 */

/*
 * Dedicated to the folks at Manex Visual Effects who used FreeBSD machines
 * to render those amazing effects in "The Matrix" movie.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/module.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/sysctl.h>

#include <machine/md_var.h>
#include <saver.h>

#include <machine/pc/display.h>

static int blanked;
static u_short *window;

static char attr[]={
	0x2f,
	0x0a,
	0x0a,
	0x0a,
	0x02,
	0x02,
	0x02,
	0x02,
	0x02,
	0x02,
	0x02,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00
};

static char counter[]={
	' ',
	' ',
	' ',
	'0',
	'0',
	'1',
	'1',
	'2',
	'2',
	'3',
	'4',
	'5',
	'6',
	'7',
	'8',
	'9'
};

#define MAX_COLOR	31
#define MAX_COUNTER	15

#define	CONSOLE_VECT(x,y) \
	(window + (y)*cur_console->xsize + (x))

static char *line;

static int
matrix_saver(video_adapter_t *adp, int blank)
{
	scr_stat *scp=cur_console;
	int gly;
	int i;

	if (blank) {
		if (adp->va_info.vi_flags & V_INFO_GRAPHICS)
			return EAGAIN;
		if (blanked == 0) {
#ifdef PC98
			if (epson_machine_id == 0x20) {
				outb(0x43f, 0x42);
				outb(0x0c17, inb(0xc17) & ~0x08);
				outb(0x43f, 0x40);
			}
#endif /* PC98 */
			window=(u_short *)adp->va_window;
			fillw(((FG_LIGHTGREY|BG_BLACK)<<8) | scr_map[0x20],
				window,scp->xsize*scp->ysize);
			set_border(scp, 0);
		}
		if (blanked++ < 2)
			return 0;
		blanked = 1;
		/* Move the contents down 1 line */
		/*bcopy(scp->scr_buf,scp->scr_buf+scp->xsize,
			scp->xsize*(scp->ysize-1)*sizeof(u_short));
		fillw(scp->term.cur_color|scr_map[0x20],scp->scr_buf,
			scp->xsize); */
		bcopy(window,window+scp->xsize,
			scp->xsize*(scp->ysize-1)*sizeof(u_short));
		fillw(scp->term.cur_color|scr_map[0x20],window,
			scp->xsize);
		/* Draw new glyphs */
		for(i=0;i<scp->xsize;i++) {
			gly=random() & 0x007f;
			
			*CONSOLE_VECT(i,1)=scr_map[gly]|(attr[line[i]]<<8);
			/* Make a single bright spot */
			if(line[i]==0) {
				line[i]++;
			} else {
				line[i]=line[i]+(gly & 0x1);
			}
			if(line[i]>MAX_COLOR) line[i]=0;
		}
		/* Draw the counter */
		for(i=0;i<=MAX_COUNTER;i++) {
			gly=random() & MAX_COUNTER;
			*CONSOLE_VECT(i,0)=scr_map[counter[gly]]|(0x02<<8);
		}
		mark_all(scp);
	} else {
#ifdef PC98
		if (epson_machine_id == 0x20) {
			outb(0x43f, 0x42);
			outb(0x0c17, inb(0xc17) | 0x08);
			outb(0x43f, 0x40);
		}
#endif /* PC98 */
		blanked = 0;
	}
	return 0;
}

static int
matrix_init(video_adapter_t *adp)
{
	int i;

	line=malloc(cur_console->xsize,M_DEVBUF,M_WAITOK);
	for(i=0; i<cur_console->xsize; i++) {
		line[i]=random() & MAX_COLOR;
	}
	blanked = 0;
	return 0;
}

static int
matrix_term(video_adapter_t *adp)
{
	free(line,M_DEVBUF);
	return 0;
}

static scrn_saver_t matrix_module = {
	"matrix_saver", matrix_init, matrix_term, matrix_saver, NULL,
};

SAVER_MODULE(matrix_saver, matrix_module);

==> matrix_4.0.tgz: matrix/matrix_saver.c <==
/*-
 * Copyright (c) 1999 Andrzej Bialecki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$Id$
 */

/*
 * Dedicated to the folks at Manex Visual Effects who used FreeBSD machines
 * to render those amazing effects in "The Matrix" movie.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/module.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/sysctl.h>
#include <sys/consio.h>
#include <sys/fbio.h>

#include <machine/pc/display.h>

#include <dev/fb/fbreg.h>
#include <dev/fb/splashreg.h>
#include <dev/syscons/syscons.h>

static int blanked;

static char attr[]={
	0x2f,
	0x0a,
	0x0a,
	0x0a,
	0x02,
	0x02,
	0x02,
	0x02,
	0x02,
	0x02,
	0x02,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x02,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x02,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00,
	0x00
};

static char counter[]={
	' ',
	' ',
	' ',
	'0',
	'0',
	'1',
	'1',
	'2',
	'2',
	'3',
	'4',
	'5',
	'6',
	'7',
	'8',
	'9'
};

#define MAX_COLOR	31
#define MAX_COUNTER	15

static char *line;

static int
matrix_saver(video_adapter_t *adp, int blank)
{
	sc_softc_t *sc;
	scr_stat *scp;
	int gly;
	int i;

	sc = sc_find_softc(adp, NULL);
	if (sc == NULL)
		return EAGAIN;
	scp = sc->cur_scp;

	if (blank) {
		if (adp->va_info.vi_flags & V_INFO_GRAPHICS)
			return EAGAIN;
		if (blanked == 0) {
#ifdef PC98
			if (epson_machine_id == 0x20) {
				outb(0x43f, 0x42);
				outb(0x0c17, inb(0xc17) & ~0x08);
				outb(0x43f, 0x40);
			}
#endif /* PC98 */
			/* clear the screen and set the border color */
			/* sc_vtb_clear(&scp->scr, sc->scr_map[0x20],
				     (FG_LIGHTGREY | BG_BLACK) << 8); */
			(*vidsw[adp->va_index]->set_hw_cursor)(adp, -1, -1);
			set_border(scp, 0);
		}
		if (blanked++ < 2)
			return 0;
		blanked = 1;
		/* Move the contents down 1 line */
		sc_vtb_ins(&sc->cur_scp->scr,0,scp->xsize,sc->scr_map[0x20],
			scp->term.cur_color);
		/* Draw new glyphs */
		for(i=0;i<scp->xsize;i++) {
			gly=random() & 0x007f;
			
			sc_vtb_putc(&sc->cur_scp->scr,i+scp->xsize,attr[line[i]]<<8,gly);
			/* Make a single bright spot */
			if(line[i]==0) {
				line[i]++;
			} else {
				line[i]=line[i]+(gly & 0x1);
			}
			if(line[i]>MAX_COLOR) line[i]=0;
		}
		/* Draw the counter */
		for(i=0;i<=MAX_COUNTER;i++) {
			gly=random() & MAX_COUNTER;
			sc_vtb_putc(&sc->cur_scp->scr,i,0x02<<8,counter[gly]);
		}
		
	} else {
#ifdef PC98
		if (epson_machine_id == 0x20) {
			outb(0x43f, 0x42);
			outb(0x0c17, inb(0xc17) | 0x08);
			outb(0x43f, 0x40);
		}
#endif /* PC98 */
		blanked = 0;
	}
	return 0;
}

static int
matrix_init(video_adapter_t *adp)
{
	sc_softc_t *sc;
	int i;

	sc = sc_find_softc(adp, NULL);
	line=malloc(sc->cur_scp->xsize,M_DEVBUF,M_WAITOK);
	for(i=0; i<sc->cur_scp->xsize; i++) {
		line[i]=random() & MAX_COLOR;
	}
	blanked = 0;
	return 0;
}

static int
matrix_term(video_adapter_t *adp)
{
	free(line,M_DEVBUF);
	return 0;
}

static scrn_saver_t matrix_module = {
	"matrix_saver", matrix_init, matrix_term, matrix_saver, NULL,
};

SAVER_MODULE(matrix_saver, matrix_module);
